/*
 * Copyright (c) 2019 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package vcl

import (
	"fmt"
	"os"
	"path/filepath"
	"regexp"
	"text/template"
)

var fMap = template.FuncMap{
	"fqVersion": func(spec Spec) string { return spec.FQVersion() },
	"vclMangle": func(s string) string { return mangle(s) },
}

var vclIllegal = regexp.MustCompile("[^[:word:]-]+")

// InitTemplates initializes templates for VCL generation.
func InitTemplates(
	tmplMap map[string]string) (map[*template.Template]string, error) {

	tmpls2files := make(map[*template.Template]string, len(tmplMap))
	for tmplPath, filePath := range tmplMap {
		if tmpl, err := template.New(filepath.Base(tmplPath)).
			Funcs(fMap).ParseFiles(tmplPath); err != nil {
			return nil, err
		} else {
			tmpls2files[tmpl] = filePath
		}
	}
	return tmpls2files, nil
}

func replIllegal(ill []byte) []byte {
	repl := []byte("_")
	for _, b := range ill {
		repl = append(repl, []byte(fmt.Sprintf("%02x", b))...)
	}
	repl = append(repl, []byte("_")...)
	return repl
}

// WriteTemplate generates a VCL file at path to implement spec, using
// the template.
func (spec Spec) WriteTemplate(tmpl *template.Template, path string) error {
	file, err := os.OpenFile(path, os.O_WRONLY|os.O_CREATE, 0644)
	if err != nil {
		return err
	}
	if err = tmpl.Execute(file, spec); err != nil {
		file.Close()
		return err
	}
	return file.Close()
}

func mangle(s string) string {
	if s == "" {
		return s
	}
	mangled := vclIllegal.ReplaceAllFunc([]byte(s), replIllegal)
	return string(mangled)
}
